# $Header: /var/lib/cvsd/var/lib/cvsd/Fuzzled/src/Fuzzled-Helper-SharedMemory/lib/Fuzzled/Helper/SharedMemory.pm,v 1.1.1.1 2007-02-14 12:20:32 timb Exp $
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# (c) Tim Brown, 2006
# <mailto:timb@nth-dimension.org.uk>
# <http://www.nth-dimension.org.uk/> / <http://www.machine.org.uk/>

package Fuzzled::Helper::SharedMemory;

use 5.008007;
use strict;
use warnings;

require Exporter;
use AutoLoader qw(AUTOLOAD);

use IPC::SysV qw(IPC_PRIVATE IPC_RMID IPC_CREAT S_IRWXU);
use IPC::Semaphore;

our @ISA = qw(Exporter);

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

# This allows declaration use Fuzzled::Helper::SharedMemory ':all';
# If you do not need this, moving things directly into @EXPORT or @EXPORT_OK
# will save memory.
our %EXPORT_TAGS = ( 'all' => [ qw(
	
) ] );

our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

our @EXPORT = qw(
	
);

our $VERSION = ('$Header: /var/lib/cvsd/var/lib/cvsd/Fuzzled/src/Fuzzled-Helper-SharedMemory/lib/Fuzzled/Helper/SharedMemory.pm,v 1.1.1.1 2007-02-14 12:20:32 timb Exp $' =~ m/.Header: .* ([\d.]+) .*/)[0];

# Preloaded methods go here.

sub new {
	my $self;
	$self = shift;
	$self = bless({}, $self);
	$self->{'memorysize'} = shift;
	$self->{'endofdatamarker'} = shift;
	$self->{'debugflag'} = shift;
	$self->{'memoryhandle'} = shmget(IPC_PRIVATE, $self->{'memorysize'}, S_IRWXU);
	if (!$self->{'memoryhandle'}) {
		$self->{'debugflag'} && print STDERR "Fuzzled::Helper::SharedMemory->new() failed\n";
		return 0;
	} 
	$self->{'semaphorehandle'} = IPC::Semaphore->new(IPC_PRIVATE, 1, S_IRWXU | IPC_CREAT);
	if (!$self->{'semaphorehandle'}) {
		$self->{'debugflag'} && print STDERR "Fuzzled::Helper::SharedMemory->new() failed\n";
		return 0;
	}
	$self->{'semaphorehandle'}->setval(0,1);
	$self->{'debugflag'} && print STDERR "Fuzzled::Helper::SharedMemory->new() succeeded\n";
	return $self;
}

sub read {
	my $self;
	my $readdata;
	$self = shift;
	$readdata = "";
	if ($self->lock() == 1) {
		$readdata = $self->peek();
		$self->unlock();
	}
	if ($readdata ne "") {
		$self->{'debugflag'} && print STDERR "Fuzzled::Helper::SharedMemory->read() succeeded: read " . length($readdata) . " byte(s)\n";
		return $readdata;
	}
	$self->{'debugflag'} && print STDERR "Fuzzled::Helper::SharedMemory->read() failed\n";
	return "";
}

sub peek {
	my $self;
	my $peekdata;
	my $returncode;
	$self = shift;
	$returncode = shmread($self->{'memoryhandle'}, $peekdata, 0, $self->{'memorysize'});
	if ($returncode == 1) {
		$peekdata = substr($peekdata, 0, index($peekdata, $self->{'endofdatamarker'}));
		$self->{'debugflag'} && print STDERR "Fuzzled::Helper::SharedMemory->peek() succeeded: read " . length($peekdata) . " byte(s)\n";
		return $peekdata;
	}
	$self->{'debugflag'} && print STDERR "Fuzzled::Helper::SharedMemory->peek() failed\n";
	return "";
}

sub write {
	my $self;
	my $writedata;
	my $returncode;
	$self = shift;
	$writedata = shift;
	if ($self->lock() == 1) {
		$returncode = $self->poke($writedata);
		$self->unlock();
	}
	if ($returncode == 1) {
		$self->{'debugflag'} && print STDERR "Fuzzled::Helper::SharedMemory->write() succeeded: wrote " . length($writedata) . " byte(s)\n";
		return 1;
	} else {
		$self->{'debugflag'} && print STDERR "Fuzzled::Helper::SharedMemory->write() failed\n";
		return 0;
	}
}

sub poke {
	my $self;
	my $pokedata;
	my $returncode;
	$self = shift;
	$pokedata = shift;
	$returncode = shmwrite($self->{'memoryhandle'}, $pokedata . $self->{'endofdatamarker'}, 0, $self->{'memorysize'});
	if ($returncode == 1) {
		$self->{'debugflag'} && print STDERR "Fuzzled::Helper::SharedMemory->poke() succeeded: wrote " . length($pokedata) . " byte(s)\n";
		return 1;
	} else {
		$self->{'debugflag'} && print STDERR "Fuzzled::Helper::SharedMemory->poke() failed\n";
		return 0;
	}
}

sub lock {
	my $self;
	my $returncode;
	$self = shift;
	$returncode = $self->{'semaphorehandle'}->op(0,-1,0);
	if ($returncode) {
		$self->{'debugflag'} && print STDERR "Fuzzled::Helper::SharedMemory->lock() succeeded\n";
		return 1;
	}
	$self->{'debugflag'} && print STDERR "Fuzzled::Helper::SharedMemory->lock() failed\n";
	return 0;
}

sub unlock {
	my $self;
	my $returncode;
	$self = shift;
	$returncode = $self->{'semaphorehandle'}->op(0,1,0);
	if ($returncode) {
		$self->{'debugflag'} && print STDERR "Fuzzled::Helper::SharedMemory->unlock() succeeded\n";
		return 1;
	}
	$self->{'debugflag'} && print STDERR "Fuzzled::Helper::SharedMemory->unlock() failed\n";
	return 0;
}

sub dealloc {
	my $self;
	$self = shift;
	shmctl($self->{'memoryhandle'}, IPC_RMID, 0);
	$self->{'semaphorehandle'}->remove();
	$self->{'debugflag'} && print STDERR "Fuzzled::Helper::SharedMemory->dealloc() succeeded\n";
	return 1;
}

# Autoload methods go after =cut, and are processed by the autosplit program.

1;
__END__

# Below is stub documentation for your module. You'd better edit it!

=head1 NAME

Fuzzled::Helper::SharedMemory - Perl extension for Fuzzled to handle the SharedMemory connection

=head1 SYNOPSIS

  use Fuzzled::Helper::SharedMemory;

  my $sharedmemorytesthandle;
  my $readdata;

  $sharedmemorytesthandle = Fuzzled::Helper::SharedMemory->new(1024, "\x03\x04", 1);
  $sharedmemorytesthandle->lock();
  $sharedmemorytesthandle->poke("Test");
  $sharedmemorytesthandle->unlock();
  $readdata = $sharedmemorytesthandle->read();
  $sharedmemorytesthandle->dealloc();

=head1 DESCRIPTION

C<Fuzzled::Helper::SharedMemory> is a Perl extension for Fuzzled to handle the shared memory.  The following methods are available:

=over 4

=item Fuzzled::Helper::SharedMemory->new(<memorysize>, <endofdatamarker>, <debugflag>)

Constructs a new C<Fuzzled::Helper::SharedMemory> object allowing access to a shared memory pool.

=item Fuzzled::Helper::SharedMemory->read()

Locks and reads from the shared memory pool.

Returns the read data.

=item Fuzzled::Helper::SharedMemory->peek()

Reads from the shared memory pool.

Returns the read data.

=item Fuzzled::Helper::SharedMemory->write()

Locks and writes to the shared memory pool.

Returns 1 on success and 0 on failure.

=item Fuzzled::Helper::SharedMemory->poke()

Writes to the shared memory pool.

Returns 1 on success and 0 on failure.

=item Fuzzled::Helper::SharedMemory->lock()

Locks the shared memory pool.

Returns 1 on success and 0 on failure.

=item Fuzzled::Helper::SharedMemory->unlock()

Unlocks the shared memory pool.

Returns 1 on success and 0 on failure.

=item Fuzzled::Helper::SharedMemory->dealloc()

Deallocates the shared memory pool.

Returns 1.

=head1 SEE ALSO

=head1 AUTHOR

(c) Tim Brown, 2006
<mailto:timb@nth-dimension.org.uk>

=head1 COPYRIGHT AND LICENSE

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

=cut