# $Header: /var/lib/cvsd/var/lib/cvsd/Fuzzled/src/Fuzzled-Protocol-HTTP/lib/Fuzzled/Protocol/HTTP.pm,v 1.1.1.1 2007-02-14 12:20:31 timb Exp $
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with self program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# (c) Tim Brown, 2006
# <mailto:timb@nth-dimension.org.uk>
# <http://www.nth-dimension.org.uk/> / <http://www.machine.org.uk/>

package Fuzzled::Protocol::HTTP;

use 5.008007;
use strict;
use warnings;

require Exporter;
use AutoLoader qw(AUTOLOAD);

use Fuzzled::Helper::SharedMemory;
use Fuzzled::Helper;
use Fuzzled::Helper::TCP;
use Fuzzled::Factory::Repeat;
use Fuzzled::Factory::Replace;
use Fuzzled::Factory::Increment;
use Fuzzled::Factory::BruteForce;
use Fuzzled::Namespace::FormatString;

our @ISA = qw(Exporter);

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

# This allows declaration use Fuzzled::Protocol::HTTP ':all';
# If you do not need self, moving things directly into @EXPORT or @EXPORT_OK
# will save memory.
our %EXPORT_TAGS = ( 'all' => [ qw(
	
) ] );

our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

our @EXPORT = qw(
	
);

our $VERSION = ('$Header: /var/lib/cvsd/var/lib/cvsd/Fuzzled/src/Fuzzled-Protocol-HTTP/lib/Fuzzled/Protocol/HTTP.pm,v 1.1.1.1 2007-02-14 12:20:31 timb Exp $' =~ m/.Header: .* ([\d.]+) .*/)[0];

# Preloaded methods go here.
sub new {
	my $self;
	$self = shift;
	$self = bless({}, $self);
	$self->{'hostname'} = shift;
	$self->{'portnumber'} = shift;
	$self->{'maximumprocess'} = shift;
	$self->{'debugflag'} = shift;
	$self->{'debugflag'} && print STDERR "Fuzzled::Protocol::HTTP->new() succeeded\n";
	return $self;
}

sub attack {
	my $self;
	my $tcphandle;
	my $currentprocess;
	my $memoryerror;
	my $consumererror;
	my $processid;
	$self = shift;
	$self->{'helperhandle'} = Fuzzled::Helper->new($self->{'debugflag'});
	$tcphandle = Fuzzled::Helper::TCP->new($self->{'hostname'}, $self->{'portnumber'}, $self->{'debugflag'});
	if ($tcphandle == 0) {
		die "Fuzzled::Helper::TCP->new() failed: unable to connect to host " . $self->{'hostname'} . ", port " . $self->{'portnumber'} . "\n";
	}
	$tcphandle->close();
	$currentprocess = 0;
	$memoryerror = 0;
	$consumererror = 0;
	while (($currentprocess < $self->{'maximumprocess'}) && ($memoryerror == 0) && ($consumererror == 0)) {
		$self->{'memoryhandle'} = Fuzzled::Helper::SharedMemory->new(4500, "\x03EOD\x04", $self->{'debugflag'});
		if ($self->{'memoryhandle'} == 0) {
			$memoryerror = 1;
		} else {
			$self->{'memoryhandle'}->write("\x03EMPTY\x04");
			$processid = $self->consumer();
			if ($processid < 0) {
				$self->{'memoryhandle'}->dealloc();
				$consumererror = 1;
			} else {
				if ($processid == 0) {
					print "[fuzzled] " . $$ . " finished\n";
					$self->{'memoryhandle'}->dealloc();
					$self->{'debugflag'} && print STDERR "Fuzzled::Protocol::HTTP->attack() succeeded: consumer\n";
					return 2;
				}
				print "[fuzzled] " . $$ . " begat " . $processid . "\n";
				$self->{'memoryhandles'}[$currentprocess] = $self->{'memoryhandle'};
				$currentprocess ++;
			}
		}
	}
	$self->{'maximumprocess'} = $currentprocess;
	if ($self->{'maximumprocess'} > 0) {
		if ($self->producer() > 0) {
			$self->{'debugflag'} && print STDERR "Fuzzled::Protocol::HTTP->attack() succeeded: producer\n";
			return 1;
		}
		$self->{'debugflag'} && print STDERR "Fuzzled::Protocol::HTTP->attack() failed: producer\n";
		return 0;
	}
	$self->{'debugflag'} && print STDERR "Fuzzled::Protocol::HTTP->attack() failed: producer\n";
	return 0;
}

sub producer {
	my $self;
	my $overflowhandle;
	my $overflowstring;
	my $rangehandle;
	my $rangestring;
	my $formathandle;
	my $formatstring;
	my $requeststring;
	my $fullrequesthandle;
	my $fullrequeststring;
	my $queuedflag;
	my $memoryhandle;
	$self = shift;
	$SIG{'INT'} = sub {
		$queuedflag = 1;
		while ($queuedflag == 1) {
			$queuedflag = 0;
			foreach $memoryhandle (@{$self->{'memoryhandles'}}) {
				if ($memoryhandle->lock() == 1) {
					if ($memoryhandle->peek() eq "\x03EMPTY\x04") {
						$memoryhandle->poke("\x03END\x04");
						$queuedflag = 1;
					} else {
						if ($memoryhandle->peek() ne "\x03END\x04") {
							$queuedflag = 1;
						}
					}
					$memoryhandle->unlock();
				}
				
			}
		}
		die "Fuzzled::Protocol::HTTP->producer() failed: interrupted\n";
	};
	print "[fuzzled] checking overflows (HEAD)\n";
	$requeststring = "HEAD / HTTP/1.0\n";
	$requeststring = $requeststring . "Host: " . $self->{'hostname'} . ":" . $self->{'portnumber'} . "\n";
	$requeststring = $requeststring . "User-Agent: Mozilla/5.0 (compatible; HTTP)\n";
	$requeststring = $requeststring . "\n";
	$overflowhandle = Fuzzled::Factory::Repeat->new("A", 0, 4096);
	while ($overflowstring = $overflowhandle->generate()) {
		$fullrequesthandle = Fuzzled::Factory::Replace->new($requeststring, [["HEAD", ""], [" /", " /"], ["Host: " . $self->{'hostname'} . ":" . $self->{'portnumber'}, "Host: "], ["User-Agent: Mozilla/5.0 \\\(compatible; HTTP\\\)", ""], ["Mozilla/5.0 \\\(compatible; HTTP\\\)", ""]], $overflowstring);
		while ($fullrequeststring = $fullrequesthandle->generate()) {
			$queuedflag = 0;
			while ($queuedflag != 1) {
				foreach $memoryhandle (@{$self->{'memoryhandles'}}) {
					if ($memoryhandle->lock() == 1) {
						if ($memoryhandle->peek() eq "\x03EMPTY\x04") {
							$memoryhandle->poke($fullrequeststring);
							$memoryhandle->unlock();
							$queuedflag = 1;
							last;
						}
						$memoryhandle->unlock();
					}
				}
				if ($queuedflag == 0) {
					$self->{'helperhandle'}->sleep(0.1);
				}
			}
		}
	}
	print "[fuzzled] checking overflows (GET)\n";
	$requeststring = "GET / HTTP/1.0\n";
	$requeststring = $requeststring . "Host: " . $self->{'hostname'} . ":" . $self->{'portnumber'} . "\n";
	$requeststring = $requeststring . "User-Agent: Mozilla/5.0 (compatible; HTTP)\n";
	$requeststring = $requeststring . "\n";
	$overflowhandle = Fuzzled::Factory::Repeat->new("A", 0, 4096);
	while ($overflowstring = $overflowhandle->generate()) {
		$fullrequesthandle = Fuzzled::Factory::Replace->new($requeststring, [["GET", ""], [" /", " /"], ["Host: " . $self->{'hostname'} . ":" . $self->{'portnumber'}, "Host: "], ["User-Agent: Mozilla/5.0 \\\(compatible; HTTP\\\)", ""], ["Mozilla/5.0 \\\(compatible; HTTP\\\)", ""]], $overflowstring);
		while ($fullrequeststring = $fullrequesthandle->generate()) {
			$queuedflag = 0;
			while ($queuedflag != 1) {
				foreach $memoryhandle (@{$self->{'memoryhandles'}}) {
					if ($memoryhandle->lock() == 1) {
						if ($memoryhandle->peek() eq "\x03EMPTY\x04") {
							$memoryhandle->poke($fullrequeststring);
							$memoryhandle->unlock();
							$queuedflag = 1;
							last;
						}
						$memoryhandle->unlock();
					}
				}
				if ($queuedflag == 0) {
					$self->{'helperhandle'}->sleep(0.1);
				}
			}
		}
	}
	print "[fuzzled] checking overflows (POST)\n";
	$requeststring = "POST / HTTP/1.0\n";
	$requeststring = $requeststring . "Host: " . $self->{'hostname'} . ":" . $self->{'portnumber'} . "\n";
	$requeststring = $requeststring . "Content-Type: application/x-www-form-urlencoded\n";
	$requeststring = $requeststring . "Content-Length: 10\n";
	$requeststring = $requeststring . "User-Agent: Mozilla/5.0 \\\(compatible; HTTP\\\)\n";
	$requeststring = $requeststring . "\n";
	$requeststring = $requeststring . "AAAAAAAAAA\n";
	$requeststring = $requeststring . "\n";
	$overflowhandle = Fuzzled::Factory::Repeat->new("A", 0, 4096);
	while ($overflowstring = $overflowhandle->generate()) {
		$fullrequesthandle = Fuzzled::Factory::Replace->new($requeststring, [["POST", ""], [" /", " /"], ["Host: " . $self->{'hostname'} . ":" . $self->{'portnumber'}, "Host: "], ["application/x-www-form-urlencoded", ""], ["Content-Length: 10", "Content-Length: "], ["User-Agent: Mozilla/5.0 \\\(compatible; HTTP\\\)", ""], ["Mozilla/5.0 \\\(compatible; HTTP\\\)", ""]], $overflowstring);
		while ($fullrequeststring = $fullrequesthandle->generate()) {
			$queuedflag = 0;
			while ($queuedflag != 1) {
				foreach $memoryhandle (@{$self->{'memoryhandles'}}) {
					if ($memoryhandle->lock() == 1) {
						if ($memoryhandle->peek() eq "\x03EMPTY\x04") {
							$memoryhandle->poke($fullrequeststring);
							$memoryhandle->unlock();
							$queuedflag = 1;
							last;
						}
						$memoryhandle->unlock();
					}
				}
				if ($queuedflag == 0) {
					$self->{'helperhandle'}->sleep(0.1);
				}
			}
		}
	}
	print "[fuzzled] checking overflows (OPTIONS)\n";
	$requeststring = "OPTIONS / HTTP/1.0\n";
	$requeststring = $requeststring . "Host: " . $self->{'hostname'} . ":" . $self->{'portnumber'} . "\n";
	$requeststring = $requeststring . "User-Agent: Mozilla/5.0 \\\(compatible; HTTP\\\)\n";
	$requeststring = $requeststring . "\n";
	$overflowhandle = Fuzzled::Factory::Repeat->new("A", 0, 4096);
	while ($overflowstring = $overflowhandle->generate()) {
		$fullrequesthandle = Fuzzled::Factory::Replace->new($requeststring, [["OPTIONS", ""], [" /", " /"], ["Host: " . $self->{'hostname'} . ":" . $self->{'portnumber'}, "Host: "], ["User-Agent: Mozilla/5.0 \\\(compatible; HTTP\\\)", ""], ["Mozilla/5.0 \\\(compatible; HTTP\\\)", ""]], $overflowstring);
		while ($fullrequeststring = $fullrequesthandle->generate()) {
			$queuedflag = 0;
			while ($queuedflag != 1) {
				foreach $memoryhandle (@{$self->{'memoryhandles'}}) {
					if ($memoryhandle->lock() == 1) {
						if ($memoryhandle->peek() eq "\x03EMPTY\x04") {
							$memoryhandle->poke($fullrequeststring);
							$memoryhandle->unlock();
							$queuedflag = 1;
							last;
						}
						$memoryhandle->unlock();
					}
				}
				if ($queuedflag == 0) {
					$self->{'helperhandle'}->sleep(0.1);
				}
			}
		}
	}
	print "[fuzzled] checking overflows (TRACK)\n";
	$requeststring = "TRACK / HTTP/1.0\n";
	$requeststring = $requeststring . "Host: " . $self->{'hostname'} . ":" . $self->{'portnumber'} . "\n";
	$requeststring = $requeststring . "User-Agent: Mozilla/5.0 \\\(compatible; HTTP\\\)\n";
	$requeststring = $requeststring . "\n";
	$requeststring = $requeststring . "AAAAAAAAAA\n";
	$requeststring = $requeststring . "\n";
	$overflowhandle = Fuzzled::Factory::Repeat->new("A", 0, 4096);
	while ($overflowstring = $overflowhandle->generate()) {
		$fullrequesthandle = Fuzzled::Factory::Replace->new($requeststring, [["TRACK", ""], [" /", " /"], ["Host: " . $self->{'hostname'} . ":" . $self->{'portnumber'}, "Host: "], ["User-Agent: Mozilla/5.0 \\\(compatible; HTTP\\\)", ""], ["Mozilla/5.0 \\\(compatible; HTTP\\\)", ""]], $overflowstring);
		while ($fullrequeststring = $fullrequesthandle->generate()) {
			$queuedflag = 0;
			while ($queuedflag != 1) {
				foreach $memoryhandle (@{$self->{'memoryhandles'}}) {
					if ($memoryhandle->lock() == 1) {
						if ($memoryhandle->peek() eq "\x03EMPTY\x04") {
							$memoryhandle->poke($fullrequeststring);
							$memoryhandle->unlock();
							$queuedflag = 1;
							last;
						}
						$memoryhandle->unlock();
					}
				}
				if ($queuedflag == 0) {
					$self->{'helperhandle'}->sleep(0.1);
				}
			}
		}
	}
	print "[fuzzled] checking overflows (TRACE)\n";
	$requeststring = "TRACE / HTTP/1.0\n";
	$requeststring = $requeststring . "Host: " . $self->{'hostname'} . ":" . $self->{'portnumber'} . "\n";
	$requeststring = $requeststring . "User-Agent: Mozilla/5.0 \\\(compatible; HTTP\\\)\n";
	$requeststring = $requeststring . "\n";
	$requeststring = $requeststring . "AAAAAAAAAA\n";
	$requeststring = $requeststring . "\n";
	$overflowhandle = Fuzzled::Factory::Repeat->new("A", 0, 4096);
	while ($overflowstring = $overflowhandle->generate()) {
		$fullrequesthandle = Fuzzled::Factory::Replace->new($requeststring, [["TRACE", ""], [" /", " /"], ["Host: " . $self->{'hostname'} . ":" . $self->{'portnumber'}, "Host: "], ["User-Agent: Mozilla/5.0 \\\(compatible; HTTP\\\)", ""], ["Mozilla/5.0 \\\(compatible; HTTP\\\)", ""]], $overflowstring);
		while ($fullrequeststring = $fullrequesthandle->generate()) {
			$queuedflag = 0;
			while ($queuedflag != 1) {
				foreach $memoryhandle (@{$self->{'memoryhandles'}}) {
					if ($memoryhandle->lock() == 1) {
						if ($memoryhandle->peek() eq "\x03EMPTY\x04") {
							$memoryhandle->poke($fullrequeststring);
							$memoryhandle->unlock();
							$queuedflag = 1;
							last;
						}
						$memoryhandle->unlock();
					}
				}
				if ($queuedflag == 0) {
					$self->{'helperhandle'}->sleep(0.1);
				}
			}
		}
	}
	print "[fuzzled] checking ranges (POST)\n";
	$requeststring = "POST / HTTP/1.0\n";
	$requeststring = $requeststring . "Host: " . $self->{'hostname'} . ":" . $self->{'portnumber'} . "\n";
	$requeststring = $requeststring . "Content-Type: application/x-www-form-urlencoded\n";
	$requeststring = $requeststring . "Content-Length: 10\n";
	$requeststring = $requeststring . "User-Agent: Mozilla/5.0 \\\(compatible; HTTP\\\)\n";
	$requeststring = $requeststring . "\n";
	$requeststring = $requeststring . "AAAAAAAAAA\n";
	$requeststring = $requeststring . "\n";
	$rangehandle = Fuzzled::Factory::Increment->new(-255, 255, 1);
	while ($rangestring = $rangehandle->generate()) {
		$fullrequesthandle = Fuzzled::Factory::Replace->new($requeststring, [["Content-Length: 10", "Content-Length: "]], $rangestring);
		while ($fullrequeststring = $fullrequesthandle->generate()) {
			$queuedflag = 0;
			while ($queuedflag != 1) {
				foreach $memoryhandle (@{$self->{'memoryhandles'}}) {
					if ($memoryhandle->lock() == 1) {
						if ($memoryhandle->peek() eq "\x03EMPTY\x04") {
							$memoryhandle->poke($fullrequeststring);
							$memoryhandle->unlock();
							$queuedflag = 1;
							last;
						}
						$memoryhandle->unlock();
					}
				}
				if ($queuedflag == 0) {
					$self->{'helperhandle'}->sleep(0.1);
				}
			}
		}
	}
	print "[fuzzled] checking format strings (HEAD)\n";
	$requeststring = "HEAD / HTTP/1.0\n";
	$requeststring = $requeststring . "Host: " . $self->{'hostname'} . ":" . $self->{'portnumber'} . "\n";
	$requeststring = $requeststring . "User-Agent: Mozilla/5.0 \\\(compatible; HTTP\\\)\n";
	$requeststring = $requeststring . "\n";
	$formathandle = Fuzzled::Factory::BruteForce->new(Fuzzled::Namespace::FormatString->generate() . ["1"]);
	while ($formatstring = $formathandle->generate()) {
		$fullrequesthandle = Fuzzled::Factory::Replace->new($requeststring, [["HEAD", ""], [" /", " /"], ["Host: " . $self->{'hostname'} . ":" . $self->{'portnumber'}, "Host: "], ["User-Agent: Mozilla/5.0 \\\(compatible; HTTP\\\)", ""], ["Mozilla/5.0 \\\(compatible; HTTP\\\)", ""]], $formatstring);
		while ($fullrequeststring = $fullrequesthandle->generate()) {
			$queuedflag = 0;
			while ($queuedflag != 1) {
				foreach $memoryhandle (@{$self->{'memoryhandles'}}) {
					if ($memoryhandle->lock() == 1) {
						if ($memoryhandle->peek() eq "\x03EMPTY\x04") {
							$memoryhandle->poke($fullrequeststring);
							$memoryhandle->unlock();
							$queuedflag = 1;
							last;
						}
						$memoryhandle->unlock();
					}
				}
				if ($queuedflag == 0) {
					$self->{'helperhandle'}->sleep(0.1);
				}
			}
		}
	}
	print "[fuzzled] checking formats (GET)\n";
	$requeststring = "GET / HTTP/1.0\n";
	$requeststring = $requeststring . "Host: " . $self->{'hostname'} . ":" . $self->{'portnumber'} . "\n";
	$requeststring = $requeststring . "User-Agent: Mozilla/5.0 \\\(compatible; HTTP\\\)\n";
	$requeststring = $requeststring . "\n";
	$formathandle = Fuzzled::Factory::BruteForce->new(Fuzzled::Namespace::FormatString->generate() . ["1"]);
	while ($formatstring = $formathandle->generate()) {
		$fullrequesthandle = Fuzzled::Factory::Replace->new($requeststring, [["GET", ""], [" /", " /"], ["Host: " . $self->{'hostname'} . ":" . $self->{'portnumber'}, "Host: "], ["User-Agent: Mozilla/5.0 \\\(compatible; HTTP\\\)", ""], ["Mozilla/5.0 \\\(compatible; HTTP\\\)", ""]], $formatstring);
		while ($fullrequeststring = $fullrequesthandle->generate()) {
			$queuedflag = 0;
			while ($queuedflag != 1) {
				foreach $memoryhandle (@{$self->{'memoryhandles'}}) {
					if ($memoryhandle->lock() == 1) {
						if ($memoryhandle->peek() eq "\x03EMPTY\x04") {
							$memoryhandle->poke($fullrequeststring);
							$memoryhandle->unlock();
							$queuedflag = 1;
							last;
						}
						$memoryhandle->unlock();
					}
				}
				if ($queuedflag == 0) {
					$self->{'helperhandle'}->sleep(0.1);
				}
			}
		}
	}
	print "[fuzzled] checking format strings (POST)\n";
	$requeststring = "POST / HTTP/1.0\n";
	$requeststring = $requeststring . "Host: " . $self->{'hostname'} . ":" . $self->{'portnumber'} . "\n";
	$requeststring = $requeststring . "Content-Type: application/x-www-form-urlencoded\n";
	$requeststring = $requeststring . "Content-Length: 10\n";
	$requeststring = $requeststring . "User-Agent: Mozilla/5.0 \\\(compatible; HTTP\\\)\n";
	$requeststring = $requeststring . "\n";
	$requeststring = $requeststring . "AAAAAAAAAA\n";
	$requeststring = $requeststring . "\n";
	$formathandle = Fuzzled::Factory::BruteForce->new(Fuzzled::Namespace::FormatString->generate() . ["1"]);
	while ($formatstring = $formathandle->generate()) {
		$fullrequesthandle = Fuzzled::Factory::Replace->new($requeststring, [["POST", ""], [" /", " /"], ["Host: " . $self->{'hostname'} . ":" . $self->{'portnumber'}, "Host: "], ["application/x-www-form-urlencoded", ""], ["Content-Length: 10", "Content-Length: "], ["User-Agent: Mozilla/5.0 \\\(compatible; HTTP\\\)", ""], ["Mozilla/5.0 \\\(compatible; HTTP\\\)", ""]], $formatstring);
		while ($fullrequeststring = $fullrequesthandle->generate()) {
			$queuedflag = 0;
			while ($queuedflag != 1) {
				foreach $memoryhandle (@{$self->{'memoryhandles'}}) {
					if ($memoryhandle->lock() == 1) {
						if ($memoryhandle->peek() eq "\x03EMPTY\x04") {
							$memoryhandle->poke($fullrequeststring);
							$memoryhandle->unlock();
							$queuedflag = 1;
							last;
						}
						$memoryhandle->unlock();
					}
				}
				if ($queuedflag == 0) {
					$self->{'helperhandle'}->sleep(0.1);
				}
			}
		}
	}
	print "[fuzzled] checking format strings (OPTIONS)\n";
	$requeststring = "OPTIONS / HTTP/1.0\n";
	$requeststring = $requeststring . "Host: " . $self->{'hostname'} . ":" . $self->{'portnumber'} . "\n";
	$requeststring = $requeststring . "User-Agent: Mozilla/5.0 \\\(compatible; HTTP\\\)\n";
	$requeststring = $requeststring . "\n";
	$formathandle = Fuzzled::Factory::BruteForce->new(Fuzzled::Namespace::FormatString->generate() . ["1"]);
	while ($formatstring = $formathandle->generate()) {
		$fullrequesthandle = Fuzzled::Factory::Replace->new($requeststring, [["OPTIONS", ""], [" /", " /"], ["Host: " . $self->{'hostname'} . ":" . $self->{'portnumber'}, "Host: "], ["User-Agent: Mozilla/5.0 \\\(compatible; HTTP\\\)", ""], ["Mozilla/5.0 \\\(compatible; HTTP\\\)", ""]], $formatstring);
		while ($fullrequeststring = $fullrequesthandle->generate()) {
			$queuedflag = 0;
			while ($queuedflag != 1) {
				foreach $memoryhandle (@{$self->{'memoryhandles'}}) {
					if ($memoryhandle->lock() == 1) {
						if ($memoryhandle->peek() eq "\x03EMPTY\x04") {
							$memoryhandle->poke($fullrequeststring);
							$memoryhandle->unlock();
							$queuedflag = 1;
							last;
						}
						$memoryhandle->unlock();
					}
				}
				if ($queuedflag == 0) {
					$self->{'helperhandle'}->sleep(0.1);
				}
			}
		}
	}
	print "[fuzzled] checking format strings (TRACK)\n";
	$requeststring = "TRACK / HTTP/1.0\n";
	$requeststring = $requeststring . "Host: " . $self->{'hostname'} . ":" . $self->{'portnumber'} . "\n";
	$requeststring = $requeststring . "User-Agent: Mozilla/5.0 \\\(compatible; HTTP\\\)\n";
	$requeststring = $requeststring . "\n";
	$requeststring = $requeststring . "AAAAAAAAAA\n";
	$requeststring = $requeststring . "\n";
	$formathandle = Fuzzled::Factory::BruteForce->new(Fuzzled::Namespace::FormatString->generate() . ["1"]);
	while ($formatstring = $formathandle->generate()) {
		$fullrequesthandle = Fuzzled::Factory::Replace->new($requeststring, [["TRACK", ""], [" /", " /"], ["Host: " . $self->{'hostname'} . ":" . $self->{'portnumber'}, "Host: "], ["User-Agent: Mozilla/5.0 \\\(compatible; HTTP\\\)", ""], ["Mozilla/5.0 \\\(compatible; HTTP\\\)", ""]], $formatstring);
		while ($fullrequeststring = $fullrequesthandle->generate()) {
			$queuedflag = 0;
			while ($queuedflag != 1) {
				foreach $memoryhandle (@{$self->{'memoryhandles'}}) {
					if ($memoryhandle->lock() == 1) {
						if ($memoryhandle->peek() eq "\x03EMPTY\x04") {
							$memoryhandle->poke($fullrequeststring);
							$memoryhandle->unlock();
							$queuedflag = 1;
							last;
						}
						$memoryhandle->unlock();
					}
				}
				if ($queuedflag == 0) {
					$self->{'helperhandle'}->sleep(0.1);
				}
			}
		}
	}
	print "[fuzzled] checking format strings (TRACE)\n";
	$requeststring = "TRACE / HTTP/1.0\n";
	$requeststring = $requeststring . "Host: " . $self->{'hostname'} . ":" . $self->{'portnumber'} . "\n";
	$requeststring = $requeststring . "User-Agent: Mozilla/5.0 \\\(compatible; HTTP\\\)\n";
	$requeststring = $requeststring . "\n";
	$requeststring = $requeststring . "AAAAAAAAAA\n";
	$requeststring = $requeststring . "\n";
	$formathandle = Fuzzled::Factory::BruteForce->new(Fuzzled::Namespace::FormatString->generate() . ["1"]);
	while ($formatstring = $formathandle->generate()) {
		$fullrequesthandle = Fuzzled::Factory::Replace->new($requeststring, [["TRACE", ""], [" /", " /"], ["Host: " . $self->{'hostname'} . ":" . $self->{'portnumber'}, "Host: "], ["User-Agent: Mozilla/5.0 \\\(compatible; HTTP\\\)", ""], ["Mozilla/5.0 \\\(compatible; HTTP\\\)", ""]], $formatstring);
		while ($fullrequeststring = $fullrequesthandle->generate()) {
			$queuedflag = 0;
			while ($queuedflag != 1) {
				foreach $memoryhandle (@{$self->{'memoryhandles'}}) {
					if ($memoryhandle->lock() == 1) {
						if ($memoryhandle->peek() eq "\x03EMPTY\x04") {
							$memoryhandle->poke($fullrequeststring);
							$memoryhandle->unlock();
							$queuedflag = 1;
							last;
						}
						$memoryhandle->unlock();
					}
				}
				if ($queuedflag == 0) {
					$self->{'helperhandle'}->sleep(0.1);
				}
			}
		}
	}
	$queuedflag = 1;
	while ($queuedflag == 1) {
		$queuedflag = 0;
		foreach $memoryhandle (@{$self->{'memoryhandles'}}) {
			if ($memoryhandle->lock() == 1) {
				if ($memoryhandle->peek() eq "\x03EMPTY\x04") {
					$memoryhandle->poke("\x03END\x04");
					$queuedflag = 1;
				} else {
					if ($memoryhandle->peek() ne "\x03END\x04") {
						$queuedflag = 1;
					}
				}
				$memoryhandle->unlock();
			}
			
		}
	}
	$self->{'debugflag'} && print STDERR "Fuzzled::Protocol::HTTP->producer() succeeded\n";
	return 1;
}

sub consumer {
	my $self;
	my $processid;
	my $requeststring;
	my $fullrequeststring;
	my $tcphandle;
	my $responsestring;
	$self = shift;
	$processid = fork();
	if ($processid == 0) {
		$SIG{'INT'} = sub {
			$self->{'debugflag'} && print STDERR "Fuzzled::Protocol::HTTP->consumer() interrupted\n";
		};
		while ($self->{'memoryhandle'}->read() ne "\x03END\x04") {
			if ($self->{'memoryhandle'}->lock() == 1) {
				$requeststring = $self->{'memoryhandle'}->peek();
				if ($requeststring eq "\x03END\x04") {
					$self->{'memoryhandle'}->unlock();
				} else {
					if ($requeststring eq "\x03EMPTY\x04") {
						$self->{'memoryhandle'}->unlock();
						$self->{'helperhandle'}->sleep(1);
					} else {
						$tcphandle = Fuzzled::Helper::TCP->new($self->{'hostname'}, $self->{'portnumber'}, $self->{'debugflag'});
						if ($tcphandle == 0) {
							die "Fuzzled::Helper::TCP->new() failed: unable to connect to host " . $self->{'hostname'} . ", port " . $self->{'portnumber'} . "\n";
						}
						$self->{'memoryhandle'}->poke("\x03EMPTY\x04");
						$self->{'memoryhandle'}->unlock();
						$fullrequeststring = $requeststring;
						$tcphandle->write($fullrequeststring);
						print "[fuzzled] wrote " . $requeststring . "\n";
						$self->{'helperhandle'}->log("HTTP." . $$, "wrote " . $fullrequeststring . "\n");
						do {
							$responsestring = $tcphandle->read();
							print "[fuzzled] read " . $responsestring . "\n";
							$self->{'helperhandle'}->log("HTTP." . $$, "read " . $responsestring . "\n");
						} while ($responsestring ne "");
						$tcphandle->close();
					}
				}		
			} else {
				$self->{'helperhandle'}->sleep(1);
			}
		}
	}
	$self->{'debugflag'} && print STDERR "Fuzzled::Protocol::HTTP->consumer() succeeded\n";
	return $processid;
}

# Autoload methods go after =cut, and are processed by the autosplit program.

1;
__END__

=head1 NAME

Perl extension for Fuzzled to fuzz HTTP

=head1 SYNOPSIS

  use Fuzzled::Protocol::HTTP;

  my $httptesthandle;

  $httptesthandle = Fuzzled::Protocol::HTTP->new("service.example.org", 80, 1, 1);
  $httptesthandle->attack();

=head1 DESCRIPTION

C<Fuzzled::Protocol::HTTP> is a Perl extension for Fuzzled to fuzz HTTP.  The following methods are available:

=over 4

=item Fuzzled::Protocol::HTTP->new(<hostname>, <portnumber>, <maximumprocess>, <debugflag>)

Constructs a new C<Fuzzled::Protocol::HTTP> object describing the connection to be fuzzed.

=item Fuzzled::Protocol::HTTP->attack()

Carries out the fuzzing.

Returns 1 

=head1 SEE ALSO

=head1 AUTHOR

(c) Tim Brown, 2006
<mailto:timb@nth-dimension.org.uk>

=head1 COPYRIGHT AND LICENSE

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with self program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

=cut