# $Header: /var/lib/cvsd/var/lib/cvsd/Fuzzled/src/Fuzzled-Protocol-HTTPFormsAuthenticate/lib/Fuzzled/Protocol/HTTPFormsAuthenticate.pm,v 1.1.1.1 2007-02-14 12:20:28 timb Exp $
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with self program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
# (c) Tim Brown, 2006
# <mailto:timb@nth-dimension.org.uk>
# <http://www.nth-dimension.org.uk/> / <http://www.machine.org.uk/>

package Fuzzled::Protocol::HTTPFormsAuthenticate;

use 5.008007;
use strict;
use warnings;

require Exporter;
use AutoLoader qw(AUTOLOAD);

use Fuzzled::Helper::SharedMemory;
use Fuzzled::Helper;
use Fuzzled::Helper::TCP;
use Fuzzled::Factory::BruteForce;
use Fuzzled::Namespace::Printable;
use Fuzzled::Factory::Increment;
use Fuzzled::Factory::Pad;

our @ISA = qw(Exporter);

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

# This allows declaration use Fuzzled::Protocol::HTTPFormsAuthenticate ':all';
# If you do not need self, moving things directly into @EXPORT or @EXPORT_OK
# will save memory.
our %EXPORT_TAGS = ( 'all' => [ qw(
	
) ] );

our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

our @EXPORT = qw(
	
);

our $VERSION = ('$Header: /var/lib/cvsd/var/lib/cvsd/Fuzzled/src/Fuzzled-Protocol-HTTPFormsAuthenticate/lib/Fuzzled/Protocol/HTTPFormsAuthenticate.pm,v 1.1.1.1 2007-02-14 12:20:28 timb Exp $' =~ m/.Header: .* ([\d.]+) .*/)[0];

# Preloaded methods go here.
sub new {
	my $self;
	$self = shift;
	$self = bless({}, $self);
	$self->{'hostname'} = shift;
	$self->{'portnumber'} = shift;
	$self->{'maximumprocess'} = shift;
	$self->{'keepaliveflag'} = shift;
	$self->{'successpattern'} = shift;
	$self->{'debugflag'} = shift;
	$self->{'debugflag'} && print STDERR "Fuzzled::Protocol::HTTPFormsAuthenticate->new() succeeded\n";
	return $self;
}

sub attack {
	my $self;
	my $tcphandle;
	my $currentprocess;
	my $memoryerror;
	my $consumererror;
	my $processid;
	$self = shift;
	$self->{'helperhandle'} = Fuzzled::Helper->new($self->{'debugflag'});
	$tcphandle = Fuzzled::Helper::TCP->new($self->{'hostname'}, $self->{'portnumber'}, $self->{'debugflag'});
	if ($tcphandle == 0) {
		die "Fuzzled::Helper::TCP->new() failed: unable to connect to host " . $self->{'hostname'} . ", port " . $self->{'portnumber'} . "\n";
	}
	$tcphandle->close();
	$currentprocess = 0;
	$memoryerror = 0;
	$consumererror = 0;
	while (($currentprocess < $self->{'maximumprocess'}) && ($memoryerror == 0) && ($consumererror == 0)) {
		$self->{'memoryhandle'} = Fuzzled::Helper::SharedMemory->new(15, "\x03EOD\x04", $self->{'debugflag'});
		if ($self->{'memoryhandle'} == 0) {
			$memoryerror = 1;
		} else {
			$self->{'memoryhandle'}->write("\x03EMPTY\x04");
			$processid = $self->consumer();
			if ($processid < 0) {
				$self->{'memoryhandle'}->dealloc();
				$consumererror = 1;
			} else {
				if ($processid == 0) {
					print "[fuzzled] " . $$ . " finished\n";
					$self->{'memoryhandle'}->dealloc();
					$self->{'debugflag'} && print STDERR "Fuzzled::Protocol::HTTPFormsAuthenticate->attack() succeeded: consumer\n";
					return 2;
				}
				print "[fuzzled] " . $$ . " begat " . $processid . "\n";
				$self->{'memoryhandles'}[$currentprocess] = $self->{'memoryhandle'};
				$currentprocess ++;
			}
		}
	}
	$self->{'maximumprocess'} = $currentprocess;
	if ($self->{'maximumprocess'} > 0) {
		if ($self->producer() > 0) {
			$self->{'debugflag'} && print STDERR "Fuzzled::Protocol::HTTPFormsAuthenticate->attack() succeeded: producer\n";
			return 1;
		}
		$self->{'debugflag'} && print STDERR "Fuzzled::Protocol::HTTPFormsAuthenticate->attack() failed: producer\n";
		return 0;
	}
	$self->{'debugflag'} && print STDERR "Fuzzled::Protocol::HTTPFormsAuthenticate->attack() failed: producer\n";
	return 0;
}

sub producer {
	my $self;
	my $username1handle;
	my $username1character;
	my $username2handle;
	my $username2character;
	my $usernamenumberhandle;
	my $usernamenumber;
	my $padhandle;
	my $padstring;
	my $queuedflag;
	my $memoryhandle;
	$self = shift;
	$SIG{'INT'} = sub {
		$queuedflag = 1;
		while ($queuedflag == 1) {
			$queuedflag = 0;
			foreach $memoryhandle (@{$self->{'memoryhandles'}}) {
				if ($memoryhandle->lock() == 1) {
					if ($memoryhandle->peek() eq "\x03EMPTY\x04") {
						$memoryhandle->poke("\x03END\x04");
						$queuedflag = 1;
					} else {
						if ($memoryhandle->peek() ne "\x03END\x04") {
							$queuedflag = 1;
						}
					}
					$memoryhandle->unlock();
				}
				
			}
		}
		die "Fuzzled::Protocol::HTTPFormsAuthenticate->producer() failed: interrupted\n";
	};
	print "[fuzzled] brute forcing users\n";
	$username1handle = Fuzzled::Factory::BruteForce->new(Fuzzled::Namespace::Printable->generate());
	while ($username1character = $username1handle->generate()) {
		$username2handle = Fuzzled::Factory::BruteForce->new(Fuzzled::Namespace::Printable->generate());
		while ($username2character = $username2handle->generate()) {
			$usernamenumberhandle = Fuzzled::Factory::Increment->new(0, 999, 1);
			while ($usernamenumber = $usernamenumberhandle->generate()) {
				$padhandle = Fuzzled::Factory::Pad->new($usernamenumber, "0", 3 - length($usernamenumber), 3 - length($usernamenumber));
				while ($padstring = $padhandle->generate()) {
					$queuedflag = 0;
					while ($queuedflag != 1) {
						foreach $memoryhandle (@{$self->{'memoryhandles'}}) {
							if ($memoryhandle->lock() == 1) {
								if ($memoryhandle->peek() eq "\x03EMPTY\x04") {
									$memoryhandle->poke($username1character . $username2character . $padstring . $usernamenumber);
									$memoryhandle->unlock();
									$queuedflag = 1;
									last;
								}
								$memoryhandle->unlock();
							}
						}
						if ($queuedflag == 0) {
							$self->{'helperhandle'}->sleep(0.1);
						}
					}
				}
			}
		}
	}
	$queuedflag = 1;
	while ($queuedflag == 1) {
		$queuedflag = 0;
		foreach $memoryhandle (@{$self->{'memoryhandles'}}) {
			if ($memoryhandle->lock() == 1) {
				if ($memoryhandle->peek() eq "\x03EMPTY\x04") {
					$memoryhandle->poke("\x03END\x04");
					$queuedflag = 1;
				} else {
					if ($memoryhandle->peek() ne "\x03END\x04") {
						$queuedflag = 1;
					}
				}
				$memoryhandle->unlock();
			}
			
		}
	}
	$self->{'debugflag'} && print STDERR "Fuzzled::Protocol::HTTPFormsAuthenticate->producer() succeeded\n";
	return 1;
}

sub consumer {
	my $self;
	my $processid;
	my $requeststring;
	my $fullrequeststring;
	my $tcphandle;
	my $responsestring;
	$self = shift;
	$processid = fork();
	if ($processid == 0) {
		$SIG{'INT'} = sub {
			$self->{'debugflag'} && print STDERR "Fuzzled::Protocol::HTTPFormsAuthenticate->consumer() interrupted\n";
		};
		while ($self->{'memoryhandle'}->read() ne "\x03END\x04") {
			if ($self->{'memoryhandle'}->lock() == 1) {
				$requeststring = $self->{'memoryhandle'}->peek();
				if ($requeststring eq "\x03END\x04") {
					$self->{'memoryhandle'}->unlock();
				} else {
					if ($requeststring eq "\x03EMPTY\x04") {
						$self->{'memoryhandle'}->unlock();
						$self->{'helperhandle'}->sleep(1);
					} else {
						if ($self->{'keepaliveflag'} == 1) {
							if ($tcphandle) {
								if ($tcphandle->eof() == 1) {
									$tcphandle = Fuzzled::Helper::TCP->new($self->{'hostname'}, $self->{'portnumber'}, $self->{'debugflag'});
									if ($tcphandle == 0) {
										die "Fuzzled::Helper::TCP->new() failed: unable to connect to host " . $self->{'hostname'} . ", port " . $self->{'portnumber'} . "\n";
									}
								}
							} else {
								$tcphandle = Fuzzled::Helper::TCP->new($self->{'hostname'}, $self->{'portnumber'}, $self->{'debugflag'});
								if ($tcphandle == 0) {
									die "Fuzzled::Helper::TCP->new() failed: unable to connect to host " . $self->{'hostname'} . ", port " . $self->{'portnumber'} . "\n";
								}
							}
						} else {
							$tcphandle = Fuzzled::Helper::TCP->new($self->{'hostname'}, $self->{'portnumber'}, $self->{'debugflag'});
							if ($tcphandle == 0) {
								die "Fuzzled::Helper::TCP->new() failed: unable to connect to host " . $self->{'hostname'} . ", port " . $self->{'portnumber'} . "\n";
							}
						}
						$self->{'memoryhandle'}->poke("\x03EMPTY\x04");
						$self->{'memoryhandle'}->unlock();
						$fullrequeststring = "POST /login.aspx HTTP/1.1\n";
						$fullrequeststring = $fullrequeststring . "Host: " . $self->{'hostname'}  . ":" . $self->{'portnumber'} . "\n";
						if ($self->{'keepaliveflag'} == 1) {
							$fullrequeststring = $fullrequeststring . "Connection: Keep-Alive\n";
						}
						$fullrequeststring = $fullrequeststring . "Content-Type: application/x-www-form-urlencoded\n";
						$fullrequeststring = $fullrequeststring . "Content-Length: " . (20 + length($requeststring)) . "\n";
						$fullrequeststring = $fullrequeststring . "User-Agent: Mozilla/5.0 (compatible; HTTPFormsAuthenticate)\n";
						$fullrequeststring = $fullrequeststring . "\n";
						$fullrequeststring = $fullrequeststring . "username=" . $requeststring . "&password=A\n";
						$fullrequeststring = $fullrequeststring . "\n";	$tcphandle->write($fullrequeststring);
						print "[fuzzled] " . $requeststring;
						$self->{'helperhandle'}->log("HTTPFormsAuthenticate." . $$, "wrote " . $fullrequeststring . "\n");
						do {
							$responsestring = $tcphandle->read();
							$self->{'helperhandle'}->log("HTTPFormsAuthenticate." . $$, "read " . $responsestring . "\n");
						} while ($responsestring !~ /([0-9]{3}) (.*)\n.*/);
						if ($responsestring =~ /($self->{'successpattern'})/) {
							print ": success (" . $1 . ")\n";
						} else {
							print ": failure\n";
						}
						if ($self->{'keepaliveflag'} == 0) {
							$tcphandle->close();
						}
					}
				}		
			} else {
				$self->{'helperhandle'}->sleep(1);
			}
		}
	}
	$self->{'debugflag'} && print STDERR "Fuzzled::Protocol::HTTPFormsAuthenticate->consumer() succeeded\n";
	return $processid;
}

# Autoload methods go after =cut, and are processed by the autosplit program.

1;
__END__

=head1 NAME

Perl extension for Fuzzled to brute force users

=head1 SYNOPSIS

  use Fuzzled::Protocol::HTTPFormsAuthenticate;

  my $httpformsauthenticatetesthandle;

  $httpformsauthenticatetesthandle = Fuzzled::Protocol::HTTPFormsAuthenticate->new("service.example.org", 80, 1, 0, ".*authenticated.*", 1);
  $httpformsauthenticatetesthandle->attack();

=head1 DESCRIPTION

C<Fuzzled::Protocol::HTTPFormsAuthenticate> is a Perl extension for Fuzzled to brute force users.  The following methods are available:

=over 4

=item Fuzzled::Protocol::HTTPFormsAuthenticate->new(<hostname>, <portnumber>, <maximumprocess>, <keepaliveflag>, <successpattern>, <debugflag>)

Constructs a new C<Fuzzled::Protocol::HTTPFormsAuthenticate> object describing the connection to be fuzzed.

=item Fuzzled::Protocol::HTTPFormsAuthenticate->attack()

Carries out the fuzzing.

Returns 1 

=head1 SEE ALSO

=head1 AUTHOR

(c) Tim Brown, 2006
<mailto:timb@nth-dimension.org.uk>

=head1 COPYRIGHT AND LICENSE

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with self program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

=cut